#![allow(dead_code)]

use std::ops::{Index, IndexMut};


struct Grid<T> {
    rows: Vec<Vec<T>>
}


impl<T> Grid<T> {
    fn new() -> Self {
        Grid { rows: vec![] }
    }

    fn with_default_values<D>(width: usize, height: usize) -> Self
    where T: Default + Clone
    {
        let rows = (0..height)
            .map(|_i| vec![Default::default(); width])
            .collect();

        Grid { rows }
    }

    fn from_rows<I>(rows: I) -> Self
    where I: Iterator<Item = Vec<T>>
    {
        Grid { rows: rows.collect() }
    }

    fn from_chars<F, E>(src: &str, conv: F) -> Result<Self, E>
    where F: Fn(char) -> Result<T, E>
    {
        let rows: Result<Vec<Vec<T>>, E> = src.lines()
            .map(|line| {
                line.chars()
                    .map(|c| conv(c))
                    .collect()
            })
            .collect();

        Ok(Grid { rows: rows? })
    }

    fn iter_rows(&self) -> impl Iterator<Item = &Vec<T>> {
        self.rows.iter()
    }
}


impl<T> Index<[usize; 2]> for Grid<T> {
    type Output = T;

    fn index(&self, idx: [usize; 2]) -> &T {
        let [x, y] = idx;
        &self.rows[y][x]
    }
}


impl<T> IndexMut<[usize; 2]> for Grid<T> {
    fn index_mut(&mut self, idx: [usize; 2]) -> &mut T {
        let [x, y] = idx;
        &mut self.rows[y][x]
    }
}
